package cn.estsh.i3plus.pojo.andon.bean;

import cn.estsh.i3plus.pojo.base.annotation.AnnoOutputColumn;
import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.beanutils.BeanUtils;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 安灯队列
 * @Reference :
 * @Author : silliter.yuan
 * @CreateDate : 2019-05-10 17:16
 * @Modify:
 **/
@Data
@Entity
@Table(name="ANDON_MANAGE_QUEUE", indexes = {
        @Index(columnList = "WORK_CENTER_CODE", name = "ANDON_INDEX_WORK_CENTER_CODE"),
        @Index(columnList = "WORK_CELL_CODE", name = "ANDON_INDEX_WORK_CELL_CODE"),
        @Index(columnList = "ORGANIZE_CODE", name = "ANDON_INDEX_ORGANIZE_CODE"),
        @Index(columnList = "ANDON_ORDER_NO", name = "ANDON_INDEX_ANDON_ORDER_NO"),
        @Index(columnList = "ALARM_CODE", name = "ANDON_INDEX_ALARM_CODE"),
        @Index(columnList = "STATUS_CODE", name = "ANDON_INDEX_STATUS_CODE"),
        @Index(columnList = "ACTION_CODE", name = "ANDON_INDEX_ACTION_CODE"),
        @Index(columnList = "EQUIPMENT_CODE", name = "ANDON_INDEX_EQUIPMENT_CODE")
})
@DynamicInsert
@DynamicUpdate
@EqualsAndHashCode(callSuper = true)
@Api("安灯队列")
public class AndonManageQueue extends BaseBean {

    @Column(name = "ANDON_ORDER_NO")
    @ApiParam(value = "安灯队列编号")
    private String andonOrderNo;

    @Column(name = "ALARM_CODE")
    @ApiParam(value = "安灯类型")
    private String alarmCode;

    @Column(name = "ACTION_CODE")
    @ApiParam(value = "安灯动作代码")
    private String actionCode;

    @Column(name = "WORK_CENTER_CODE")
    @ApiParam(value = "工作中心代码")
    private String workCenterCode;

    @Column(name = "WORK_CENTER_NAME_RDD")
    @ApiParam(value = "工作中心名称")
    private String workCenterNameRdd;

    @Column(name = "WORK_CELL_NAME_RDD")
    @ApiParam(value = "工作单元名称")
    private String workCellNameRdd;

    @Column(name = "WORK_CELL_CODE")
    @ApiParam(value = "工作单元代码")
    private String workCellCode;

    @Column(name = "EQUIPMENT_CODE")
    @ApiParam(value = "设备代码")
    private String equipmentCode;

    @Column(name = "EQUIPMENT_NAME_RDD")
    @ApiParam(value = "设备名称")
    private String equipmentNameRdd;

    @Column(name = "SHIFT_CODE")
    @ApiParam(value = "班次代码")
    private String shiftCode;

    @Column(name = "SHIFT_NAME_RDD")
    @ApiParam(value = "班次名称")
    private String shiftNameRdd;

    @Column(name = "STATUS_CODE")
    @ApiParam(value = "安灯状态代码")
    private String statusCode;

    @Column(name = "SEQ")
    @ApiParam(value = "序号", example = "1")
    private Double seq;

    @Column(name = "PRIORITY_LEVEL")
    @ApiParam(value = "优先级别", example = "1")
    private Integer priorityLevel;

    @Column(name="CALL_TIME")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="呼叫时间",example = "2018-01-01 01:00:00")
    @AnnoOutputColumn(hidden = true)
    public String callTime;

    @Column(name="CLOSE_TIME")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="撤销时间",example = "2018-01-01 01:00:00")
    @AnnoOutputColumn(hidden = true)
    public String closeTime;

    @Transient
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="呼叫时间，查询开始时间", example = "2018-01-01 01:00:00")
    public String callTimeStart;

    @Transient
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="呼叫时间，查询结束时间", example = "2018-01-01 01:00:00")
    public String callTimeEnd;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name="CONFIRM_TIME")
    @ApiParam(value = "响应时间", example = "2018-01-01 01:00:00")
    @AnnoOutputColumn(hidden = true)
    public String confirmTime;

    @Column(name="RESET_TIME")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="解决时间",example = "2018-01-01 01:00:00")
    @AnnoOutputColumn(hidden = true)
    public String resetTime;

    @Column(name = "CALL_USER")
    @ApiParam(value = "呼叫人")
    private String callUser;

    @Column(name = "CLOSE_USER")
    @ApiParam(value = "撤销人")
    private String closeUser;

    @Transient
    @ApiParam(value = "呼叫人名字")
    private String callUserName;

    @Column(name = "CONFIRM_USER")
    @ApiParam(value = "响应人")
    private String confirmUser;

    @Transient
    @ApiParam(value = "响应人名字")
    private String confirmUserName;

    @Column(name = "RESET_USER")
    @ApiParam(value = "解决人")
    private String resetUser;

    @Transient
    @ApiParam(value = "解决人名字")
    private String resetUserName;

    @Column(name = "IS_SHIFT_CALL")
    @ApiParam(value = "是否转呼", example = "1")
    private Integer isShiftCall;

    @Column(name="SHIFT_CALL_TIME")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiParam(value="转呼时间",example = "2018-01-01 01:00:00")
    @AnnoOutputColumn(hidden = true)
    public String shiftCallTime;

    @Column(name = "RP_OBJECT_CODE")
    @ApiParam(value = "转呼对象代码")
    private String rpObjectCode;

    @Column(name = "SC_RESPONSE_CODE")
    @ApiParam(value = "转呼响应方式")
    private String scResponseCode;

    @Column(name = "SC_RP_LEVEL")
    @ApiParam(value = "转呼通知等级")
    private String scRpLevel;

    @Column(name = "AC_CODE")
    @ApiParam(value = "呼叫原因代码")
    private String acCode;

    @Column(name = "AC_NAME_RDD")
    @ApiParam(value = "呼叫原因描述")
    private String acNameRdd;

    @Column(name = "AC_DESC")
    @ApiParam(value = "呼叫具体原因")
    private String acDesc;

    @Column(name = "EC_CODE")
    @ApiParam(value = "事件原因代码")
    private String ecCode;

    @Column(name = "EC_NAME_RDD")
    @ApiParam(value = "事件原因描述")
    private String ecNameRdd;

    @Column(name = "EC_DESC")
    @ApiParam(value = "事件具体原因")
    private String ecDesc;

    @Column(name = "EM_CODE")
    @ApiParam(value = "事件方法代码")
    private String emCode;

    @Column(name = "EM_NAME_RDD")
    @ApiParam(value = "事件方法描述")
    private String emNameRdd;

    @Column(name = "CLOSE_CAUSE")
    @ApiParam(value = "撤销原因")
    private String closeCause;

    @Column(name = "EM_DESC")
    @ApiParam(value = "事件具体方法")
    private String emDesc;

    @Column(name = "IS_HALT")
    @ApiParam(value = "是否停机", example = "1")
    private Integer isHalt;

    @Column(name = "HALT_CODE")
    @ApiParam(value = "停机类型")
    private String haltCode;

    @Column(name = "HALT_DESC")
    @ApiParam(value = "停机具体原因")
    private String haltDesc;

    @Transient
    @ApiParam(value = "异常时长", example = "0")
    private Long execptionTime;

    @Transient
    @ApiParam(value = "响应时长", example = "0")
    private Long signTime;

    @Transient
    @ApiParam(value = "ANDON次数（用于报表统计）", example = "0")
    private Long andonCount;

    @Transient
    @ApiParam(value = "平均响应时长", example = "0")
    private Double avgSignTime;

    @Transient
    @ApiParam(value = "平均解决时长", example = "0")
    private Double avgResolveTime;

    @Transient
    @ApiParam(value = "查询条件：ANDON状态")
    private List<Object> statusList;

    @Transient
    @ApiParam(value = "比例", example = "0")
    private Double proportion;

    @Transient
    @ApiParam(value = "当前比例（累加）", example = "0")
    private Double currentProportion;

    @Transient
    @ApiParam(value = "总次数（用于报表统计）", example = "0")
    private Integer sumCount;

    @Column(name = "SEND_FLAG")
    @ApiParam(value = "发送标志")
    private String sendFlag;

    /**
     * 处理类型：10=标准处理，20=自处理
     */
    @Column(name = "DISPOSAL_TYPE")
    @ApiParam(value = "处理类型")
    private String disposalType;

    /**
     * 1 = 是，2 = 否
     */
    @Transient
    @ApiParam(value = "是否勾选加入自处理问题库", example = "2")
    private Integer isDisposal;

    @Column(name = "ALARM_DETAIL_CODE")
    @ApiParam(value = "安灯子类型代码")
    private String alarmDetailCode;

    @Column(name = "ALARM_DETAIL_NAME_RDD")
    @ApiParam(value = "安灯子类型名称")
    private String alarmDetailNameRdd;

    @Column(name = "EPM_CODE")
    @ApiParam(value = "事件现象代码")
    private String epmCode;

    @Column(name = "EPM_NAME_RDD")
    @ApiParam(value = "事件现象描述")
    private String epmNameRdd;

    @Column(name = "IMPORTANCE_CODE")
    @ApiParam(value = "重要等级代码")
    private String importanceCode;

    @Column(name = "IMPORTANCE_NAME_RDD")
    @ApiParam(value = "重要等级名称")
    private String importanceNameRdd;

    @Column(name = "ORGANIZE_NAME_RDD")
    @ApiParam(value = "组织名称")
    private String organizeNameRdd;

    @Column(name = "REJECT_OPINION")
    @ApiParam(value = "驳回意见")
    private String rejectOpinion;

    @Transient
    @ApiParam(value = "安灯状态集合")
    private List<String> statusCodeList;

    // 是否转呼
    public Integer getIsShiftCall() {
        return this.isShiftCall == null ? 0 : this.isShiftCall;
    }

    // 获取优先级别
    public Integer getPriorityLevel() {
        return this.priorityLevel == null ? 0 : this.priorityLevel;
    }

    /**
     * 工位统计 构造函数
     * @param workCenterCode
     * @param workCellCode
     * @param equipmentCode
     * @param shiftCode
     * @param alarmCode
     * @param andonCount
     */
    public AndonManageQueue(String workCenterCode,String workCenterNameRdd, String workCellCode,String workCellNameRdd, String equipmentCode, String equipmentNameRdd, String shiftCode, String shiftNameRdd, String alarmCode, Long andonCount) {
        this.workCenterCode = workCenterCode;
        this.workCenterNameRdd = workCenterNameRdd;
        this.workCellNameRdd = workCellNameRdd;
        this.workCellCode = workCellCode;
        this.equipmentCode = equipmentCode;
        this.equipmentNameRdd = equipmentNameRdd;
        this.shiftCode = shiftCode;
        this.shiftNameRdd = shiftNameRdd;
        this.alarmCode = alarmCode;
        this.andonCount = andonCount;
    }

    /**
     * 原因统计 使用Java内存计算（获取原始数据）
     * @param workCenterCode
     * @param workCenterNameRdd
     * @param workCellCode
     * @param workCellNameRdd
     * @param alarmCode
     * @param ecCode
     * @param ecNameRdd
     * @param emCode
     * @param emNameRdd
     * @param callTime
     * @param confirmTime
     * @param resetTime
     */
    public AndonManageQueue(String workCenterCode,String workCenterNameRdd,String workCellCode,String workCellNameRdd,String alarmCode,String ecCode,String ecNameRdd,String emCode,String emNameRdd, String callTime,String confirmTime,String resetTime){
        this.workCenterCode = workCenterCode;
        this.workCenterNameRdd = workCenterNameRdd;
        this.workCellCode = workCellCode;
        this.workCellNameRdd = workCellNameRdd;
        this.alarmCode = alarmCode;
        this.ecCode = ecCode;
        this.ecNameRdd = ecNameRdd;
        this.emCode = emCode;
        this.emNameRdd = emNameRdd;
        this.callTime = callTime;
        this.confirmTime = confirmTime;
        this.resetTime = resetTime;
    }

    /**
     * 人员统计 使用Java内存计算（获取原始数据）
     * @param resetUser
     * @param alarmCode
     * @param callTime
     * @param confirmTime
     * @param resetTime
     */
    public AndonManageQueue(String organizeCode,String workCenterNameRdd,String workCellNameRdd,String resetUser,String alarmCode,String callTime, String confirmTime,String resetTime){
        this.organizeCode = organizeCode;
        this.workCenterNameRdd = workCenterNameRdd;
        this.workCellNameRdd = workCellNameRdd;
        this.resetUser = resetUser;
        this.alarmCode = alarmCode;
        this.callTime = callTime;
        this.confirmTime = confirmTime;
        this.resetTime = resetTime;
    }

    /**
     * 统计分析柏拉图 使用JAVA内存计算（获取原始数据）
     * @param alarmCode
     * @param ecCode
     * @param ecNameRdd
     * @param callTime
     * @param confirmTime
     * @param resetTime
     */
    public AndonManageQueue(String workCenterNameRdd,String alarmCode,String ecCode, String ecNameRdd, String callTime, String confirmTime,String resetTime){
        this.workCenterNameRdd = workCenterNameRdd;
        this.ecNameRdd = ecNameRdd;
        this.ecCode = ecCode;
        this.alarmCode = alarmCode;
        this.callTime = callTime;
        this.confirmTime = confirmTime;
        this.resetTime = resetTime;
    }

    public AndonManageQueue() {
    }

    /**
     * 安灯问题情况报表
     * @param organizeNameRdd
     * @param workCenterNameRdd
     * @param andonCount
     */
    public AndonManageQueue(String organizeNameRdd, String workCenterNameRdd, Long andonCount) {
        this.organizeNameRdd = organizeNameRdd;
        this.workCenterNameRdd = workCenterNameRdd;
        this.andonCount = andonCount;
    }

    public static void main(String[] args) {
        try {
            List ds = new ArrayList();
            ds.add(new AndonManageQueue());



            System.out.println(BeanUtils.describe(ds));
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
    }
}
